/* Mprog.cpp : Monitor Test Patterns by Jed Margolin  Windows Version 1.0

   Copyright 2001 Jed Margolin

   Permision is given to individuals to use this code for their own use
   as long as credit is given to the author.

   Any commercial use requires permission of the author
*/

#include "stdafx.h"
#include <stdio.h>			// Header File For Standard Input/Output
#include <windows.h>		// Header File For Windows
#include <math.h>			// Header File for Math Functions
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>		// Header File For The Glaux Library
#include "mprog.h"			// my include file

extern HDC			hDC;		// Private GDI Device Context
extern HGLRC		hRC;		// Permanent Rendering Context
extern HWND		    hWnd;		// Holds Our Window Handle
extern HINSTANCE	hInstance;	// Holds The Instance Of The Application

extern LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
extern GLvoid ReSizeGLScene(GLsizei,GLsizei);
extern int InitGL(GLvoid);
extern GLvoid KillGLWindow(GLvoid);
extern BOOL CreateGLWindow(char*,int,int,int,bool);
extern glRasterPos(float,float);

extern bool	active;		// Window Active Flag
extern bool	fullscreen;	// Fullscreen Flag
extern int kbhit,key;

int mscale, bscale;
double sin_table[DEG+2];		// DEG is defined in mprog.h
int lsense = 8;		// scales the speed and sensitivity of the lissajous pattern
char quit=FALSE ;
char first_update=TRUE ;
int main_sel = 0;
int hvflag = 0;				// used by HV Regulation timer

void sine_program(int,int), rsine_program(int,int);
void sine_program_pts(int,int), rsine_program_pts(int,int);
void init_sintable(void);
double wsin(int);
extern GLvoid glPrint(const char *fmt, ...);   // Where The Printing Happens
void gotoxy(int,int);

extern struct ctable lctable[8];
//------------------
struct nctable		// color table for lissajous pattern
{
	float r,g,b;	
};

	
struct nctable nlctable[8]=
{
	{ 0.0f,0.0f,0.0f }, // color 0  Black
	{ 1.0f,0.0f,0.0f },	// color 1  Red
	{ 0.0f,1.0f,0.0f },	// color 2  Green
	{ 0.0f,0.0f,1.0f },	// color 3  Blue
	{ 0.0f,1.0f,1.0f },	// color 4  Cyan
	{ 1.0f,0.0f,1.0f },	// color 5  Magenta
	{ 1.0f,1.0f,0.0f },	// color 6  Yellow
	{ 1.0f,1.0f,1.0f },	// color 7  White
};	
//----------------------------------------

#define VK_PAGEUP 33
#define VK_PAGEDN 34

int DrawGLScene(GLvoid);    // for testing
int DrawGLScene1(GLvoid);	// Lissajous Pattern #1
int DrawGLScene2(GLvoid);	// Lissajous Pattern #2
int DrawGLScene3(GLvoid);	// Color Bars
int DrawGLScene4(GLvoid);	// Crosshatch Pattern
int DrawGLScene5(GLvoid);	// Dot Pattern
void draw_menu();
//=============================================================================
//=============================================================================
// Lissajous Pattern (Connect the Lines)
int DrawGLScene1(GLvoid)				// Here's Where We Do All The Drawing
{

    glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();					// Reset the current Modelview Matrix
	glTranslatef(0.0f,0.0f,-6.0f);			

    if(bscale>0)	
	{ 
	   sine_program(mscale,bscale);		// sine program
    }
	else
	{
	   rsine_program(mscale,bscale);	// reverse axis sine program
	}

	return(TRUE);						// Keep Going
}
//=============================================================================
// Lissajous Pattern (Points-only, for really slow CPUs. Not used in Windows version)
int DrawGLScene2(GLvoid)				 // Here's Where We Do All The Drawing
{
    glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();					 // Reset The Current Modelview Matrix
	glTranslatef(0.0f,0.0f,-6.0f);			

    if(bscale>0)	
	{ 
	   sine_program_pts(mscale,bscale);	 // sine program
    }
	else
	{
	   rsine_program_pts(mscale,bscale); // reverse axis sine program
	}

	return TRUE;						// Keep Going
}
//=============================================================================
// Color Bars
int DrawGLScene3(GLvoid)				// Here's Where We Do All The Drawing
{
   int n;
//-------------------
// data: color bars #1
 
   float width1 = 10.0f/6.4f;
   
   float xy1[8] = { -5.60f, 4.0f, 
		            -5.60f+width1, 4.0f,
			        -5.60f+width1,-2.0f,
				    -5.60f, -2.0f} ;
   
   int color1[8] = { 0,7,6,4,2,5,1,3 };
//--------------------
// data: color bars #2

   float width2 = 10.0f/6.4f;
   float xy2[8] = { -5.60f, -2.0f, 
		            -5.60f+width2, -2.0f,
			        -5.60f+width2,-2.8f,
			        -5.60f,-2.8f }; 				

   int color2[7] = { 3,0,5,0,4,0,7 };
//-------------------
// data: color bars #3
   float width3 = 3.65f;
   float xy3[8] = { -5.60f, -2.7f, 
		            -5.60f+width3, -2.7f,
			        -5.60f+width3,-3.8f,
			        -5.60f,-3.8f }; 				

   int color3[7] = { 1,2,3,7,0,0,0 };
//-----------------
// Draw the color bars

    glClear(GL_COLOR_BUFFER_BIT);			// Clear Screen
	glLoadIdentity();						// Reset The Current Modelview Matrix
	glTranslatef(0.0f,0.0f,-10.0f);					
				
// Draw color Bars #1
	for(n=1; n<8; n++)
	{
 	   glColor3f(nlctable[color1[n]].r,
			     nlctable[color1[n]].g,
				 nlctable[color1[n]].b);

	   glBegin(GL_QUADS);					// Draw A Quad
	   glVertex2f(xy1[0], xy1[1]);			// Top Left
	   glVertex2f(xy1[2], xy1[3]);			// Top Right
	   glVertex2f(xy1[4], xy1[5]);			// Bottom Right
	   glVertex2f(xy1[6], xy1[7]);			// Bottom Left
	   glEnd();								// Done Drawing The Quad

       xy1[0] = xy1[0] + width1;
       xy1[2] = xy1[2] + width1;
	   xy1[4] = xy1[4] + width1;
	   xy1[6] = xy1[6] + width1;
    }

// Draw Color Bars #2
	for(n=0; n<7; n++)
	{
 	   glColor3f(nlctable[color2[n]].r,
			     nlctable[color2[n]].g,
				 nlctable[color2[n]].b);

	   glBegin(GL_QUADS);					// Draw A Quad
	   glVertex2f(xy2[0], xy2[1]);			// Top Left
	   glVertex2f(xy2[2], xy2[3]);			// Top Right
	   glVertex2f(xy2[4], xy2[5]);			// Bottom Right
	   glVertex2f(xy2[6], xy2[7]);			// Bottom Left
	   glEnd();								// Done Drawing The Quad

       xy2[0] = xy2[0] + width2;
       xy2[2] = xy2[2] + width2;
	   xy2[4] = xy2[4] + width2;
	   xy2[6] = xy2[6] + width2;
    }	

// Draw Color Bars #4
	for(n=0; n<3; n++)
	{
 	   glColor3f(nlctable[color3[n]].r,
			     nlctable[color3[n]].g,
				 nlctable[color3[n]].b);

	   glBegin(GL_QUADS);					// Draw A Quad
	   glVertex2f(xy3[0], xy3[1]);			// Top Left
	   glVertex2f(xy3[2], xy3[3]);			// Top Right
	   glVertex2f(xy3[4], xy3[5]);			// Bottom Right
	   glVertex2f(xy3[6], xy3[7]);			// Bottom Left
	   glEnd();								// Done Drawing The Quad

       xy3[0] = xy3[0] + width3;
       xy3[2] = xy3[2] + width3;
	   xy3[4] = xy3[4] + width3;
	   xy3[6] = xy3[6] + width3;
    }	

	return(TRUE);							// Keep Going
}
//=============================================================================
// Crosshatch Pattern
int DrawGLScene4(GLvoid)				// Here's Where We Do All The Drawing
{
   int n;

 // data: horizontal line
   float xy1[4] = { -7.0f, 4.5f, 
				     7.0f, 4.5f };

// data: vertical lines   
   float xy2[4] = {-7.0f, 4.5f,
				   -7.0f, -4.5f }; 
//-----------------
    glClear(GL_COLOR_BUFFER_BIT);		// Clear Screen
	glLoadIdentity();					// Reset The Current Modelview Matrix
	glTranslatef(0.0f,0.0f,-12.0f);					
				
// Draw horizontal lines
	for(n=0; n<16; n++)
	{
	   glBegin(GL_LINES);				
	   glVertex2f(xy1[0], xy1[1]);		   // Top Left
	   glVertex2f(xy1[2], xy1[3]);		   // Top Right
	   glEnd();							

       xy1[1] = xy1[1] - 0.6f;
       xy1[3] = xy1[3] - 0.6f;
    }

// Draw vertical lines
	for(n=0; n<25; n++)
	{
	   glBegin(GL_LINES);				
	   glVertex2f(xy2[0], xy2[1]);		// Top Left
	   glVertex2f(xy2[2], xy2[3]);		// Top Right
	   glEnd();							

       xy2[0] = xy2[0] + 0.583f;
       xy2[2] = xy2[2] + 0.583f;
    }
	return(TRUE);						// Keep Going
}
//=============================================================================
//=============================================================================
// Dots nice
int DrawGLScene5(GLvoid)				// Here's Where We Do All The Drawing
{
   int m,n;
   float x,y;


   glClear(GL_COLOR_BUFFER_BIT);		// Clear Screen
   glLoadIdentity();					// Reset The Current Modelview Matrix
   glTranslatef(0.0f,0.0f,-12.0f);					

   y = 4.5f;				
   for(m=0; m<16; m++)			// vertical dots
   {
     x = -7.0f;
	 for(n=0; n<25; n++)		// horizontal dots
	 {
	    glBegin(GL_POINTS);		// Draw Points
	    glVertex2f(x,y);
	    glEnd();				// Done Drawing

        x = x + 0.58f;
     }
     y = y - 0.59f;
   }

   return(TRUE);				// Keep Going
}
//=============================================================================
//=============================================================================
// Make the sine table
void init_sintable(void)
{
   double rads;
   int n;

   rads = 3.14159/180.0*360.0/DEG;
   for(n=0;n<DEG;n++)
   {
      sin_table[n] =  sin(n*rads)*2.0;	
   }

   bscale = 2;
   mscale = 1;
}
//=============================================================================
// lissajous sine program for ratios greater than 1
// lsense scales the speed and sensitivity of the pattern
void sine_program(int mscale, int bscale)
{
   int i;
   double newx,newy,oldx,oldy;
   static int m;
   int speed;

   speed = mtable[main_sel].speed;

   oldx = wsin(0);						// sx 
   oldy = wsin(m/speed);				// sy 

   for(i=1;i<DEG+1;i++)	
   {
      newx = wsin(i);					// sx
      newy = wsin(bscale*i+m/speed);	// sy

	  glBegin(GL_LINES);		
	  glVertex2f(oldx,oldy);
	  glVertex2f(newx,newy);
	  glEnd();

      oldx = newx;
      oldy = newy; 
   }
   
   m = m + mscale;
   m = m & (speed*DEG-1); 
}
//=============================================================================
// lissajous sine program for ratios less than 1
// lsense scales the speed and sensitivity of the pattern
void rsine_program(int mscale, int bscale)
{
   int i;
   double newx,newy,oldx,oldy;
   static int m;
   int speed;

   speed = mtable[main_sel].speed;

   oldy = wsin(0);						// sx 
   oldx = wsin(m/speed);				// sy 

   for(i=1;i<DEG+1;i++)	// DEG+1	
   {
      newy = wsin(i);					// sx 
      newx = wsin(-bscale*i+m/speed);	// sy 

	  glBegin(GL_LINES);				
	  glVertex2f(oldx,oldy);
	  glVertex2f(newx,newy);
	  glEnd();

      oldx = newx;
      oldy = newy; 
   }
   
   m = m + mscale;
   m = m & (speed*DEG-1); 
}
//=============================================================================
// Wrap the sine table index and look it up
double wsin(int n)
{
   n = n & (DEG-1);
   return(sin_table[n]);
}
//=============================================================================
//=============================================================================
void sine_program_pts(int mscale, int bscale)
{
   int i;
   double newx,newy;
   static int m;

   for(i=0;i<DEG;i++)	
   {
      newx = wsin(i);					// sx
      newy = wsin(bscale*i+m);			// sy

	  glBegin(GL_POINTS);				
	  glVertex2f(newx,newy);
	  glEnd();

    }
   
   m = m + mscale;
   m = m & (DEG-1); 
}
//=============================================================================
void rsine_program_pts(int mscale, int bscale)
{
   int i;
   double newx,newy;
   static int m;

   for(i=0;i<DEG;i++)	
   {
      newy = wsin(i);					// sx
      newx = wsin(-bscale*i+m);			// sy

	  glBegin(GL_POINTS);				
	  glVertex2f(newx,newy);
	  glEnd();
   }
   
   m = m + mscale;
   m = m & (DEG-1); 
}
//=============================================================================
//=============================================================================
// Lissajous Patern
int lissajous_pattern()
{
   BOOL done;
   int state;
   static int color=GREEN, bkcolor=BLACK;


   done = FALSE;	

   if(kbhit)					// Was key pressed?
   {
      kbhit = FALSE;			// Reset the flag 

	  switch(key)			    // look at the key pressed
	  { 
	     case VK_ESCAPE:
		 { 
		    done = TRUE;		// ESC Signalled A Quit
			color = GREEN;		// default color for next time	
			bkcolor = BLACK;	// default background color for next time;
			break;
		 }

		 case VK_RETURN:		// Was RETURN Pressed?
		 {
		    done=TRUE;			// RETURN Signalled A Quit
			color = GREEN;		// default color for next time	
			bkcolor = BLACK;	// default background color for next time;		
			break;
		 }

		 case VK_LEFT:			// Lissajous phase
		 { 
		     mscale = mscale-1;
		     break;
		 }

		 case VK_RIGHT:			// Lissajous phase
		 { 
		     mscale = mscale+1;
		     break;
		 }

		 case VK_UP:			// Lissajous frequency ratio
		 { 
		     bscale++;
		     if(bscale==0)  { bscale = 2;}
		     break;
		 }
				
		 case VK_DOWN:			// Lissajous frequency ratio
		 { 
		     bscale--;
		     if(bscale==0)   { bscale = -2; }
		     break;
		 }
   
		 case VK_PAGEUP:		// color
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    color++;				// Lissajous color			
			    color = color & 0x07;
			 }
		     else
			 {
			    bkcolor++;				// Background color
			    bkcolor = bkcolor & 0x07;
					    
			 }
						
		     break;
		 }
	
	
		 case VK_PAGEDN:				// Lissajous color
		 { 
		    state = GetKeyState(VK_CONTROL);
		    if(state>=0)
			{
			   color--;					// Lissajous color
			   color = color & 0x07;
			}
		    else
			{
		 	   bkcolor--;				// Background color
			   bkcolor = bkcolor & 0x07;
		    
			}
						   
			break;
		 }

		 case VK_F1:				// Is F1 Being Pressed?
		 {
		    KillGLWindow();						// Kill Our Current Window
		    fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
		    // Recreate Our OpenGL Window
		    if(fullscreen)
			{ 
			   if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",640,480,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		    else
			{
		       if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",600,400,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		}

	  } // end switch
   } // end kbhit

   glClearColor(lctable[bkcolor].r,		// background color
				lctable[bkcolor].g,
				lctable[bkcolor].b,
				0.5f);			

   glColor3f(lctable[color].r,			// pattern color
				lctable[color].g,
				lctable[color].b);	
   
   DrawGLScene1();						// Draw The Scene

   return(done);
}
//=============================================================================
// Color Bars
int cbars_pattern()
{
   BOOL done;

   done = FALSE;	

   if(kbhit)					// Was key pressed?
   {
      kbhit = FALSE;			// Reset the flag 

	  switch(key)			    // look at the key pressed
	  { 
	     case VK_ESCAPE:
		 { 
		    done = TRUE;		// ESC Signalled A Quit
		    break;
		 }

		 case VK_RETURN:		// Was RETURN Pressed?
		{
		     done=TRUE;			// RETURN Signalled A Quit
		     break;
		 }

		 case VK_F1:				// Is F1 Being Pressed?
		 {
		    KillGLWindow();						// Kill Our Current Window
		    fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
		    // Recreate Our OpenGL Window
		    if(fullscreen)
			{ 
			   if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",640,480,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		    else
			{
		       if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",600,400,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		}

	  } // end switch
   } // end kbhit

   DrawGLScene3();					// Draw The Scene

   return(done);
}
//=============================================================================
// Crosshatch Pattern
int chatch_pattern()
{
   BOOL done;
   int state;
   static int color=WHITE, bkcolor=BLACK;

   done = FALSE;	

   if(kbhit)					// Was key pressed?
   {
      kbhit = FALSE;			// Reset the flag 

	  switch(key)			    // look at the key pressed
	  { 
	     case VK_ESCAPE:
		 { 
		    done = TRUE;		// ESC Signalled A Quit
			color = WHITE;		// default color for next time				
			bkcolor = BLACK;	// default background color for next time;
		    break;
		 }

		 case VK_RETURN:		// Was RETURN Pressed?
		 {
		    done=TRUE;			// RETURN Signalled A Quit
			color = WHITE;		// default color for next time	
			bkcolor = BLACK;	// default background color for next time;
		    break;
		 }

		 case VK_PAGEUP:		// color
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    color++;						// Crosshatch color		
			    color = color & 0x07;
			 }
		     else
			 {
			    bkcolor++;						// Background color
			    bkcolor = bkcolor & 0x07;
			    glClearColor(lctable[bkcolor].r,
					  	     lctable[bkcolor].g,
						     lctable[bkcolor].b,
						     0.5f);					    
			 }
						
		     break;
		 }
	
	
		 case VK_PAGEDN:		// color
		 { 
		    state = GetKeyState(VK_CONTROL);
		    if(state>=0)
			{
			   color--;					// Crosshatch color		
			   color = color & 0x07;
			}
		    else
			{
		 	   bkcolor--;				// Background color
			   bkcolor = bkcolor & 0x07;
			   glClearColor(lctable[bkcolor].r,
						    lctable[bkcolor].g,
						    lctable[bkcolor].b,
						    0.5f);					    
			}
						   
			break;
		 }

		 case VK_F1:				// Is F1 Being Pressed?
		 {
		    KillGLWindow();						// Kill Our Current Window
		    fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
		    // Recreate Our OpenGL Window
		    if(fullscreen)
			{ 
			   if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",640,480,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		    else
			{
		       if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",600,400,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		}

	  } // end switch
   } // end kbhit
 
   glClearColor(lctable[bkcolor].r,		// background color
				lctable[bkcolor].g,
				lctable[bkcolor].b,
				0.5f);			

   glColor3f(lctable[color].r,			// Crosshatch color
				lctable[color].g,
				lctable[color].b);	
 
   DrawGLScene4();						// Draw The Scene

   return(done);
}
//=============================================================================
// Dot Pattern
int dot_pattern()
{
   BOOL done;
   int state;
   static int color=WHITE, bkcolor=BLACK;


   done = FALSE;	
   if(kbhit)					// Was key pressed?
   {
      kbhit = FALSE;			// Reset the flag 

	  switch(key)			    // look at the key pressed
	  { 
	     case VK_ESCAPE:
		 { 
		    done = TRUE;		// ESC Signalled A Quit
			color = WHITE;		// default color for next time	
			bkcolor = BLACK;	// default background color for next time;
			break;
		 }

		 case VK_RETURN:		// Was RETURN Pressed?
		 {
		    done=TRUE;			// RETURN Signalled A Quit
			color = WHITE;		// default color for next time	
			bkcolor = BLACK;	// default background color for next time;
			break;
		 }

		 case VK_PAGEUP:		// color
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    color++;						// Dot Color			
			    color = color & 0x07;
			 }
		     else
			 {
			    bkcolor++;						// Background Color
			    bkcolor = bkcolor & 0x07;
			 }
						
		     break;
		 }
	
	
		 case VK_PAGEDN:		// color
		 { 
		    state = GetKeyState(VK_CONTROL);
		    if(state>=0)
			{
			   color--;						// Dot color			
			   color = color & 0x07;
			}
		    else
			{
		 	   bkcolor--;					// Background color
			   bkcolor = bkcolor & 0x07;
			}
						   
			break;
		 }

		 case VK_F1:				// Is F1 Being Pressed?
		 {
		    KillGLWindow();						// Kill Our Current Window
		    fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
		    // Recreate Our OpenGL Window
		    if(fullscreen)
			{ 
			   if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",640,480,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		    else
			{
		       if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",600,400,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		}

	  } // end switch
   } // end kbhit

   glClearColor(lctable[bkcolor].r,		// background color
				lctable[bkcolor].g,
				lctable[bkcolor].b,
				0.5f);			

   glColor3f(lctable[color].r,			// pattern color
				lctable[color].g,
				lctable[color].b);	
   
   DrawGLScene5();						// Draw The Scene

   return(done);
}
//=============================================================================
void  CALLBACK timethis(HWND hwnd,UINT message,UINT iTimerID, DWORD dwTime)
{
	hvflag = ~hvflag;		// toggle the flag
}

// HV Regulation Test
int hvreg_test()
{
   BOOL done;
   int state;
   static int color1=WHITE, color2=BLACK,first=1;
   static long counter = 0;
   done = FALSE;

   if(first)
   {
	  SetTimer(hWnd,101,250,timethis );		// set timer for 250 ms
	  first = 0;
   }

   if(kbhit)						// Was key pressed?
   {
      kbhit = FALSE;				// Reset the flag 

	  switch(key)					// look at the key pressed
	  { 
	     case VK_ESCAPE:
		 { 
		    done = TRUE;			// ESC Signalled A Quit
			color1 = WHITE;			// default color for next time	
			color2 = BLACK;			// default color for next time;
			KillTimer(hWnd,101);	// kill the timer
			first = 1;
			break;
		 }

		 case VK_RETURN:			// Was RETURN Pressed?
		 {
		    done=TRUE;				// RETURN Signalled A Quit
			color1 = WHITE;			// default color for next time	
			color2 = BLACK;			// default color for next time;
			KillTimer(hWnd,101);	// kill the timer
			first = 1;
			break;
		 }

		 case VK_PAGEUP:			// color
		 { 
		     state =  GetKeyState(VK_CONTROL);
		     if(state>=0)
			 {
			    color1++;					// color1			
			    color1 = color1 & 0x07;
			 }
		     else
			 {
			    color2++;					// color2
			    color2 = color2 & 0x07;
			 }
						
		     break;
		 }
	
	
		 case VK_PAGEDN:		// color
		 { 
		    state = GetKeyState(VK_CONTROL);
		    if(state>=0)
			{
			   color1--;						// color1			
			   color1 = color1 & 0x07;
			}
		    else
			{
		 	   color2--;						// color2
			   color2 = color2 & 0x07;
			}
						   
			break;
		 }

		 case VK_F1:				// Is F1 Being Pressed?
		 {
		    KillGLWindow();						// Kill Our Current Window
		    fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
		    // Recreate Our OpenGL Window
		    if(fullscreen)
			{ 
			   if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",640,480,16,fullscreen))
			   { return(TRUE); }				// Quit If Window Was Not Created
			}
		    else
			{
		       if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",600,400,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}

			first = 1;  // we will need to restart the timer next time	
		 }

	  } // end switch
   } // end kbhit


   // toggle between colors
   if(hvflag)
   {
	  glClearColor(lctable[color1].r,		// background color
				   lctable[color1].g,
				   lctable[color1].b,
				   0.5f);			

	  glColor3f(lctable[color2].r,			// grid color
				   lctable[color2].g,
				   lctable[color2].b);			
   }
   else
   {
	  glClearColor(lctable[color2].r,		// color
				   lctable[color2].g,
				   lctable[color2].b,
				   0.5f);			
 
	  glColor3f(lctable[color1].r,			// grid color
				   lctable[color1].g,
				   lctable[color1].b);			
   }


   DrawGLScene4();						// Draw The Scene (Crosshatch) 

//   glClear(GL_COLOR_BUFFER_BIT);		// Set Screen to clear color

   return(done);
}
//=============================================================================
// help screen
int help()
{
   BOOL done;
   int i,color;

   done = FALSE;	
   if(kbhit)					// Was key pressed?
   {
      kbhit = FALSE;			// Reset the flag 

	  switch(key)			    // look at the key pressed
	  { 
	     case VK_ESCAPE:
		 { 
		    done = TRUE;		// ESC Signalled A Quit
		    break;
		 }

		 case VK_RETURN:		// Was RETURN Pressed?
		 {
		     done=TRUE;			// RETURN Signalled A Quit
		     break;
		 }

		 case VK_F1:				// Is F1 Being Pressed?
		 {
		    KillGLWindow();						// Kill Our Current Window
		    fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
		    // Recreate Our OpenGL Window
		    if(fullscreen)
			{ 
			   if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",640,480,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		    else
			{
		       if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",600,400,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		}

	  } // end switch
   } // end kbhit

// directions
   glLoadIdentity();						// Reset The Current Modelview Matrix
   glTranslatef(0.0f,0.0f,-12.0f);	
   glClear(GL_COLOR_BUFFER_BIT);			// Clear Screen

   for(i=0; i<NUM_INSTR; i++)
   {
     color = dirmsgs[i].color;
	 glColor3f(lctable[color].r, lctable[color].g,lctable[color].b);

     gotoxy(dirmsgs[i].xpos,dirmsgs[i].ypos);
     glPrint(dirmsgs[i].msg);			// Print GL Text To The Screen
   }

   return(done);
}
//=============================================================================
int myprogram()
{
   BOOL done;
   static BOOL pattern_done=TRUE, exit_flag=FALSE;

   done = FALSE;	

   if(pattern_done)     { draw_menu();}
   else 	         
   { 
	   pattern_done = (mtable[main_sel].pgm());
       if(pattern_done && main_sel==0)  { return(1); }  // main exit
       return(done);
   }

   if(kbhit)					// Was key pressed?
   {
      kbhit = FALSE;			// Reset the flag 

	  switch(key)			    // look at the key pressed
	  { 
	     case VK_ESCAPE:
		 {
		    return(1);
		 }

		 case VK_UP:
		 {
			main_sel -= 1;
	        if(main_sel<0)   { main_sel = NUM_ENTRIES - 1; }
	        draw_menu();
			break;
		 }

		 case VK_DOWN:
		 {
			main_sel += 1;
	        main_sel %= NUM_ENTRIES;
	        draw_menu();
			break;
		 }

		 case VK_RETURN:
		 {
            pattern_done = FALSE;	// start pattern on next go-around
			return(done);
		 }

		 case VK_F1:				// Is F1 Being Pressed?
		 {
		    KillGLWindow();						// Kill Our Current Window
		    fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
		    // Recreate Our OpenGL Window
		    if(fullscreen)
			{ 
			   if (!CreateGLWindow("Monitor Patterns by Jed Margolin",640,480,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		    else
			{
		       if (!CreateGLWindow("Monitor Test Patterns by Jed Margolin",600,400,16,fullscreen))
			   { return(TRUE);}					// Quit If Window Was Not Created
			}
		 }

      }		// end switch
   }	// end kbhit		

   return(done);
}
//=============================================================================
//=============================================================================
// Redraw the Main Menu 
void draw_menu()
{
   int i;

   glLoadIdentity();						// Reset The Current Modelview Matrix
   glTranslatef(0.0f,0.0f,-12.0f);	
   glClearColor(0.0f,0.0f,0.0f,0.5f);		// black background
   glClear(GL_COLOR_BUFFER_BIT);			// Clear Screen

   //   print_header();

   for (i=0; i<NUM_ENTRIES; i++)
   {
      if(i!= main_sel)   { glColor3f(0.0f,1.0f,0.0f);}
      else			     { glColor3f(1.0f,1.0f,0.0f);}	
      gotoxy(mtable[i].xpos,mtable[i].ypos);
	  glPrint(mtable[i].msg);			// Print GL Text To The Screen
   }

   glColor3f(1.0f,1.0f,1.0f);

   gotoxy(XM1,18);
   glPrint("Press UP Arrow and DOWN Arrow to navigate the menu.");
 
   gotoxy(XM1,19);
   glPrint("Press Enter to make a selection.");

   gotoxy(XM1,20);
   glPrint("Press F1 to toggle between full screen and window.");

}
//=============================================================================
// main exit
int mexit()
{
   return(1);
}
//=============================================================================
// Translate rows and columns to screen positions
void gotoxy(int xp, int yp)
{
   float xpos,ypos;
   xpos = -6.5f+xp/6.0f;		
   ypos = +5.5f-(yp/2.4f);		
   glRasterPos2f(xpos,ypos);	
}
//=============================================================================
// Test the text functions
int DrawGLScene(GLvoid)					// Here's Where We Do All The Drawing
{
    int val = 42;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
	glLoadIdentity();									// Reset The Current Modelview Matrix
	glTranslatef(0.0f,0.0f,-1.0f);						// Move One Unit Into The Screen

	glColor3f(1.0f,1.0f,1.0f);
	glRasterPos2f(-0.45f,+0.05f);						// Position The Text On The Screen
	glPrint("Active OpenGL Text With NeHe - %d", val);	// Print GL Text To The Screen
										
	return TRUE;										// Everything Went OK
}
//=============================================================================
